{-
 *  Programmer:	Piotr Borek
 *  E-mail:     piotrborek@op.pl
 *  Copyright 2015 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-}

module Coin.UI.TagsComboBox (
    tagsComboBoxNew,
    tagsComboBoxSelect
) where

import qualified Graphics.UI.Gtk as Gtk
import Graphics.UI.Gtk ( AttrOp(..) )

import qualified Data.Text as T
import Control.Monad

import Database.Persist
import Data.List
import Coin.DB.Tables
import Coin.DB.Functions
import Coin.UI.Utils.Observable
import Coin.UI.MainState

tagsComboBoxNew :: MainState -> IO Gtk.ComboBox
tagsComboBoxNew mainState = do
    c <- Gtk.comboBoxNewWithEntry
    void $ Gtk.comboBoxSetModelText c
    (Just entry) <- Gtk.binGetChild c
    Gtk.set (Gtk.castToEntry entry) [Gtk.entryEditable := False]

    observableRegister (mainStateTagsUpdated mainState) $ tagsComboBoxUpdate' c

    Gtk.widgetSetSizeRequest c 240 28
    return c

tagsComboBoxSelect :: IO [Entity TagsTable]
tagsComboBoxSelect = runDB $ selectList [] []

tagsComboBoxUpdate' :: Gtk.ComboBox -> [Entity TagsTable] -> IO ()
tagsComboBoxUpdate' combo entities = do
    boxModel <- Gtk.comboBoxGetModelText combo

    let tagsList = sort $ flip map entities $ \(Entity _ tag) -> T.pack (tagsTableName tag)

    Gtk.listStoreClear boxModel
    forM_ tagsList $ Gtk.listStoreAppend boxModel
    Gtk.comboBoxSetActive combo 0
